//A very simple opengl/GLUT program
#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <iostream>
#include <vector>
#include <assert.h>
#include <time.h>

// rotation stuff

GLfloat objectM[4][4] = {
  {1.0, 0.0, 0.0, 0.0},
  {0.0, 1.0, 0.0, 0.0},
  {0.0, 0.0, 1.0, 0.0},
  {0.0, 0.0, 0.0, 1.0}
};

int surface_texture;
int mode;
int ice_mode;
bool draw_branches;
int crystal_mode;

// 3 = blue, 4 = red, 5 = green
int cur_col;

float blend_on;

GLfloat *trackballM = (GLfloat *)objectM;

float g_theta, rx, ry, rz;

int startx, starty, endx, endy;
float angle;

//data strucutre to store 3d points
typedef struct {
  float x;
  float y;
  float z;  
} Vector3;

Vector3 axis;

Vector3 cross(Vector3 a, Vector3 b)
{
	Vector3 c;
	c.x = a.y * b.z - a.z * b.y;
	c.y = a.z * b.x - a.x * b.z;
	c.z = a.x * b.y - a.y * b.x;
	return c;
}

float dot (Vector3 a, Vector3 b)
{
	return a.x * b.x + a.y * b.y + a.z * b.z;
}


float mag(Vector3 a)
{
	return sqrtf(a.x * a.x + a.y * a.y + a.z * a.z);
}


Vector3 unit(Vector3 a)
{
	Vector3 c;
	c.x = a.x / sqrtf(a.x * a.x + a.y * a.y + a.z * a.z);
	c.y = a.y / sqrtf(a.x * a.x + a.y * a.y + a.z * a.z);
	c.z = a.z / sqrtf(a.x * a.x + a.y * a.y + a.z * a.z);
	return c;
}

/*some texture mapping stuff */
typedef struct Image {
  unsigned long sizeX;
  unsigned long sizeY;
  char *data;
} Image;

Image *TextureImage;
int ImageLoad(char *filename, Image *image);
GLvoid LoadTexture(char* image_file, int tex_id);
void init_tex();

GLUquadricObj* mySphere;

typedef struct RGB {
  GLubyte r;
  GLubyte g; 
  GLubyte b;
} RGB;

RGB myimage[64][64];
RGB* pixel;

bool mouse_down;
float counter;
float cur_x, cur_y;

bool done_growing;

using namespace std;

double PI = 3.14159265358979323846;

//global variables - necessary evil when using GLUT
//global width and height
int GW;
int GH;
//current mouse position in pixel coordinate
int x;
int y;

float p2w_x(int x) {
	return (float)(x - (GW - 1) / 2) / (GH / 2);
}

float p2w_y(int y) {
	return (float)(y - (GH - 1) / 2) / (GH / 2);
}


void rotate()
{
	Vector3 a, b, c;
	a.x = p2w_x(startx);// - tx / 2 - cx / 2;
	a.y = p2w_y(starty);// - ty / 2 - cy / 2;

	float a_z = 1 - ((a.x * a.x) + (a.y * a.y));

	if (a_z > 0)
	{
		a.z = sqrtf(a_z);
	}
	else
	{
		
		a.x = a.x / sqrtf((a.x * a.x) + (a.y * a.y));
		a.y = a.y / sqrtf((a.x * a.x) + (a.y * a.y));
		a.z = 0;
	}

	b.x = p2w_x(endx);// - tx / 2 - cx / 2;
	b.y = p2w_y(endy);// - ty / 2 - cy / 2;
	
	
	float b_z = 1 - ((a.x * a.x) + (a.y * a.y));

	if (b_z > 0)
	{
		b.z = sqrtf(b_z);
	}
	else
	{
		b.x = b.x / sqrtf((b.x * b.x) + (b.y * b.y));
		b.y = b.y / sqrtf((b.x * b.x) + (b.y * b.y));
		b.z = 0;
	}

	c = cross(a, b);

	float m_a = sqrtf(dot(a, a));
	float m_b = sqrtf(dot(b, b));

	float theta = acosf (dot(a, b) / (mag(a) * mag(b)) );

	g_theta = 360 * theta/3.14159265358979323846;
	
	rx = c.x;
	ry = c.y;
	rz = c.z;
}

//set up some materials
typedef struct materialStruct {
  GLfloat ambient[4];
  GLfloat diffuse[4];
  GLfloat specular[4];
  GLfloat shininess[1];
} materialStruct;

materialStruct surface1 = {
  {0.3, 0.0, 0.0, 0.6},
  {0.9, 0.0, 0.0, 0.6},
  {0.0, 0.0, 0.0, 0.6},
  {0.0}
};
materialStruct surface2 = {
  {0.8, 0.8, 0.8, 0.6},
  {1.0, 1.0, 1.0, 0.6},
  {0.0, 0.0, 0.0, 0.6},
  {0.0}
};

materialStruct surface3 = {
  {0.0, 0.3, 0.7, 1.0},
  {0.0, 0.9, 0.7, 1.0},
  {0.0, 0.0, 0.0, 1.0},
  {100.0}
};

materialStruct surface4 = {
  {1.0, 1.0, 1.0, 1.0},
  {1.0, 1.0, 1.0, 1.0},
  {0.0, 0.0, 0.0, 1.0},
  {100.0}
};


//sets up a specific material
void materials(materialStruct materials) {
  glMaterialfv(GL_FRONT, GL_AMBIENT, materials.ambient);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, materials.diffuse);
  glMaterialfv(GL_FRONT, GL_SPECULAR, materials.specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, materials.shininess);
}



float yt;
float adj;
Vector3 p2w(float x, float y)
{

	Vector3 c;
	yt = 0.0;
	yt = (3 * ((y  - (GH - 1) / 2) / (GH / 2))) / 1.733333;
	if (yt > 1.0)
	{
		adj = (yt - 1.0) * 0.8 + 0.4;
	}
	else
	{
		adj = 0.4;
	}

	c.y = yt + 0.1184972 - ((0.1184972) * cosf(yt / 1.5 )) + ((0.1184972 * adj) * sinf( fabs((yt / 1.5 ))));

	//c.y = ((3 * ((y  - (GH - 1) / 2) / (GH / 2))) / 1.733333) + 0.1184972 - 
	//	((0.1184972) * (1.5 - fabsf((((3 * ((y  - (GH - 1) / 2) / (GH / 2))) / 1.733333)))));
	//c.y = ((y - (GH - 1) / 2) / (GH / 2));
	c.z = -(c.y / 3) - .5;
	c.x = ((x - (GW - 1) / 2) / (GH / 2)) * (0.09380725 * c.y  + 1.742134) ;
	//c.z = 0;

	return c;
}



GLfloat cameraM[4][4] = {
  {1.0, 0.0, 0.0, 0.0},
  {0.0, 1.0, 0.0, 0.0},
  {0.0, 0.0, 1.0, 0.0},
  {0.0, 0.0, 0.0, 1.0}
};

GLfloat *camM = (GLfloat *)cameraM;

/*********************************************
*
*		Node Class and Related Elements
*
**********************************************/

typedef struct vector2 {
  float x;
  float y;
} vector2;

typedef struct colorRGB {
  float R;
  float G;
  float B;
} colorRGB;

//Node class definition
class Node
{
//protected:
public:
	float scale[15];
	float maxScale;
	Vector3 trans;
	Node *last;
	float tilt;
	int type;
	float recurs;
	int color;

public:
	Node();
	Node(float i, float j); 
	float getX();
	float getY();
	float getZ();
	float getScale();
	float getMaxScale();
	float getRecurs();
	int getColor();

	int getType();
	Vector3 getTrans();
	
	void setColor(int c);
	void setRecurs(float i);
	void augScale(float s);
	void augMaxScale(float m);
	void setScale(float s);
	void setColor( float R, float G, float B );
	void setMouse_2(vector2 two);
	void setLast(Node *l);
	Node * getLast();
	float getTilt();
	void setType();
};

//Node class methods
Node::Node()
{
	color = cur_col;
	recurs = 0.0;
	scale[0] = 0.001;
	scale[1] = 0.001;
	scale[2] = 0.001;
	scale[3] = 0.001;
	scale[4] = 0.001;
	scale[5] = 0.001;
	scale[6] = 0.001;
	scale[7] = 0.001;
	scale[8] = 0.001;
	scale[9] = 0.001;
	scale[10] = 0.001;
	scale[11] = 0.001;
	scale[12] = 0.001;
	scale[13] = 0.001;
	scale[14] = 0.001;


	maxScale = 0.2;
	trans.x = 0.0;
	trans.y = 0.0;
	trans.z = 0.0;
	tilt = 0.0;
	type = ice_mode;
}

Node::Node(float i, float j)
{
	color = cur_col;
	scale[0] = 0.001;
	scale[1] = 0.001;
	scale[2] = 0.001;
	scale[3] = 0.001;
	scale[4] = 0.001;
	scale[5] = 0.001;
	scale[6] = 0.001;
	scale[7] = 0.001;
	scale[8] = 0.001;
	scale[9] = 0.001;
	scale[10] = 0.001;
	scale[11] = 0.001;
	scale[12] = 0.001;
	scale[13] = 0.001;
	scale[14] = 0.001;
	recurs = 0.0;
	maxScale = 0.2;

	Vector3 temp = p2w(i, j);
	trans.x = temp.x;
	trans.y = temp.y;
	trans.z = temp.z;
	type = ice_mode;
	
	//make icicle node tilt to the side;
	//gives value between -16.4 and 16.4
	tilt = ((float)rand()/1000 - 16.4)/1.5;
}

int Node::getColor()
{
	return color;
}

void Node::setColor(int c)
{
	color = c;
}

float Node::getRecurs()
{
	return recurs;
}

void Node::setRecurs(float i)
{	
	if (recurs < i)
	{
		//printf("recur is currently at %f\n", recurs);
		recurs = recurs + 1.0;
		//printf("recur is now at %f\n", recurs);
	}
}

void Node::setLast(Node *l)
{
	last = l;
}

float Node::getTilt()
{
	return tilt;
}

Node * Node::getLast()
{
	return last;
}


float Node::getX()
{
	//return x;
	return trans.x;
}

float Node::getY()
{
	//return y;
	return trans.y;
}

float Node::getZ()
{
	//return Z;
	return trans.z;
}

float Node::getScale()
{
	return scale[0];
}

void Node::setScale(float s)
{
	scale[0] = s;
}

float Node::getMaxScale()
{
	return maxScale;
}

void Node::augMaxScale(float m)
{
	maxScale = maxScale + m;
}

void Node::augScale(float s)
{
	scale[0] = scale[0] + s;
}


// end Node classes

Node *last_n;
Node *cur_n;


//resolution of circle

// keep track of drawn nodes
vector<Node *> nodes;

//keep track of the number of points stored thus far
int num_pts;
//keep track of which drawing mode we are in points or square
//mode ==0 means draw square otherwise draw points

// keep track of color
//colorRGB cur_col;

//state of polygon

bool shape_done;
bool settings_change;


//helper drawing routines

/*
float p2w_x(int x) {
	return (float)(x - (GW - 1) / 2) / (GH / 2);
}

float p2w_y(int y) {
	return (float)(y - (GH - 1) / 2) / (GH / 2);
}
*/

//#######################################################################
//
//		lighting

//globals for lighting - use a white light and apply materials
GLfloat light_pos[4] = {-3.0, 3.0, 2.5, 1.0};
GLfloat light_amb[4] = {0.6, 0.6, 0.6, 1.0};
GLfloat light_diff[4] = {0.6, 0.6, 0.6, 1.0};
GLfloat light_spec[4] = {0.8, 0.8, 0.8, 1.0};

void init_lighting() {

	  glEnable(GL_LIGHT0);
		//set the light's position
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
		//set up the diffuse, ambient and specular components for the light
		glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diff);
		glLightfv(GL_LIGHT0, GL_AMBIENT, light_amb);
		glLightfv(GL_LIGHT0, GL_SPECULAR, light_spec);
		//specify our lighting model as 1 normal per face

//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  
  //if (display_mode == 0)
	glShadeModel(GL_FLAT);
  //else
	//glShadeModel(GL_SMOOTH);


  glEnable(GL_NORMALIZE);
}



//the display call back - all drawing should be done in this function

void frac(Node *n, float scale)
{
	float theta = 0;
	glPushMatrix();
	glRotatef(n->tilt, 0.0, 1.0, 0.0);
	gluCylinder(mySphere, scale * n->scale[0]/8, 0, scale * n->scale[0]/2, 4, 1);
	glPopMatrix();
}

float depth = .24;

void frac3(Node *n, float scale, float k);

void frac3(Node *n, float scale, float k)
{
	if(scale > depth && (n->maxScale * scale > .12))
	for (int i = 0; i < 6; i ++)
	{
		//redraw along branch

		for (int j = 0; j < n->scale[(int)k] * 3 - .25; j++)
		{		
			//printf("recursions = %f\n", n->getRecurs());
			glPushMatrix();

			glTranslatef(0.0, 0.0, ((j + 1) * scale *  .35));

			glRotatef(90, 1.0, 0.0, 0.0);

			glPushMatrix();
			glRotatef(i * 60, 0.0, 1.0, 0.0);
			glRotatef(-30, 0.0, 0.0, 1.0);
			glRotatef(-90, 0.0, 1.0, 0.0);
			//gluCylinder(mySphere, n->scale/16, 0, n->maxScale, 4, 1);
			gluCylinder(mySphere, ( scale/(j+1.0) ) * n->scale[(int)(j+1.0)]/16, 0, 
														( 1* scale/ (j/2+1.0) ) * n->scale[(int)(j+1.0)], 4, 1);
			//printf("accessing scale at %f\n", k+1.0);

			frac3(n, scale/2, (k+1.0));
			glPopMatrix();

			glPopMatrix();
			n->setRecurs( (j + 1) );
		}
	}
}

void frac2(Node *n, float scale)
{
	//printf("scale = %f, %f, %f, %f\n", n->scale[0], n->scale[1], n->scale[2], n->scale[3]);
	float theta = 0;

	//top
	for (int i = 0; i < 4; i ++)
	{
		glPushMatrix();
		glRotatef(i * 60, 0.0, 1.0, 0.0);
		glRotatef(-30, 0.0, 0.0, 1.0);
		glRotatef(-90, 0.0, 1.0, 0.0);
		gluCylinder(mySphere, scale * n->scale[0]/32, 0, scale * n->scale[0]/1.5, 4, 1);

		frac3(n, scale/2.5, 0);
		
		glPopMatrix();
	}
	
	//bottom
	for (int i = 0; i < 4; i ++)
	{	
		glPushMatrix();
		glRotatef(i * 60, 0.0, -1.0, 0.0);
		glRotatef(-30, 0.0, 0.0, 1.0);
		glRotatef(90, 0.0, 1.0, 0.0);
		gluCylinder(mySphere, scale * n->scale[0]/32, 0, scale * n->scale[0]/1.5, 4, 1);

		frac3(n, scale/2.5, 0);

		glPopMatrix();
	}
}

void frac4(Node *n, float scale)
{
	//printf("scale = %f, %f, %f, %f\n", n->scale[0], n->scale[1], n->scale[2], n->scale[3]);
	float theta = 0;

	//top
	for (int i = 0; i < 2; i ++)
	{
		glPushMatrix();
		glRotatef(i * 180, 0.0, 1.0, 0.0);
		glRotatef(-30, 0.0, 0.0, 1.0);
		glRotatef(-90, 0.0, 1.0, 0.0);
		gluCylinder(mySphere, scale * n->scale[0]/32, 0, scale * n->scale[0]/1.5, 4, 1);

		frac3(n, scale/2.5, 0);
		
		glPopMatrix();
	}
	
	//bottom
	for (int i = 0; i < 2; i ++)
	{	
		glPushMatrix();
		glRotatef(i * 180, 0.0, -1.0, 0.0);
		glRotatef(-30, 0.0, 0.0, 1.0);
		glRotatef(90, 0.0, 1.0, 0.0);
		gluCylinder(mySphere, scale * n->scale[0]/32, 0, scale * n->scale[0]/1.5, 4, 1);

		frac3(n, scale/2.5, 0);

		glPopMatrix();
	}
}

float distance(Node *a, Node *b)
{
	float d = 0.0;
	d = (b->trans.x -a->trans.x)*(b->trans.x -a->trans.x) + (b->trans.y -a->trans.y)*(b->trans.y -a->trans.y);
	return sqrtf(d);
}

void draw_background()
{
	materials(surface4);
	Vector3 normal;
	normal.x = 0.0;
	normal.y = 0.5;
	normal.z = 1.0;

	normal = unit(normal);
	glBindTexture(GL_TEXTURE_2D, 1);
	glNormal3f(normal.x, normal.y, normal.z);
	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_POLYGON);
	
	//top
	glTexCoord2f(0.0, 1.0);
	glVertex3f(-2.7, 2.7, -10);

	glTexCoord2f(1.0, 1.0);
	glVertex3f(2.7, 2.7, -10);
	//bottom

	glTexCoord2f(1.0, 0.0);
	glVertex3f(2.7, -2.7, -10);

	glTexCoord2f(0.0, 0.0);
	glVertex3f(-2.7, -2.7, -10);

	glEnd();
}

void draw_plane()
{
	materials(surface4);
	Vector3 normal;
	normal.x = 0.0;
	normal.y = 0.5;
	normal.z = 1.0;

	normal = unit(normal);
	if (surface_texture)
		glBindTexture(GL_TEXTURE_2D, 8);
	else
		glBindTexture(GL_TEXTURE_2D, 2);
	glNormal3f(normal.x, normal.y, normal.z);
	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_POLYGON);
	
	//top
	glTexCoord2f(0.0, 1.0);
	glVertex3f(-1.5, 1.5, -1.0);

	glTexCoord2f(1.0, 1.0);
	glVertex3f(1.5, 1.5, -1.0);
	//bottom

	glTexCoord2f(1.0, 0.0);
	glVertex3f(1.5, -1.5, 0.0);

	glTexCoord2f(0.0, 0.0);
	glVertex3f(-1.5, -1.5, 0.0);

	glEnd();
}


	Node * it;

void drawNodePoints()
{
	//Node * it;

	glPointSize(4.0);
  	glBegin(GL_POINTS);
	glColor3f(0.0, 1.0, 0.0);		
		
	for (int i = 0; i < nodes.size(); i++)
	{
		it = nodes.at(i);
		
		glVertex3f(it->getX(), it->getY(), it->getZ());
		printf("dot drawn at %f, %f, %f\n", it->getX(), it->getY(), it->getZ());

	}
	glEnd();	

}


  
  float circle_res = 10;
  float theta_inc = 2 * PI / circle_res;

  float incr = 0.001;
  float deg = 0.04;

    float phi = 19.47122 -1.0400;

void drawIce(){

	float scale = 1;
	
	float dx = 0.0;
	float dy = 0.0;
	float orient = 0.0;
	glMatrixMode(GL_MODELVIEW);

	Node * last;
	glColor3f(1.0, 1.0, 1.0);
	materials(surface2);

		
for (int i = 0; i < nodes.size(); i++)
	{
		if (i == 0)
		{
			last = nodes.at(0);
		}
		else
		{
			last = it;
		}
		it = nodes.at(i);
		glBindTexture(GL_TEXTURE_2D, it->color);
		
		
		glPushMatrix();
		//draw the glut sphere behind the mesh

		glTranslatef(it->trans.x, it->trans.y, it->trans.z);
		//printf("phi %f", phi);
		glRotatef(phi, -1, 0, 0);

		//orienting ice node to previous ice node

		dx = it->trans.x - it->last->trans.x;
		dy = it->trans.y - it->last->trans.y;
		
		if (dx != 0)
		{
			orient = fabs(atanf(dy/dx) * 180/PI);

			if (dx >= 0 && dy >= 0)
			{
				orient = 90 - orient;

			}
			else if (dx >= 0 && dy < 0)
			{
				orient = 90 + orient;
			}
			else if (dx < 0 && dy >= 0)
			{
				//do nothing;
				orient = 270 + orient;
			}
			else if (dx < 0 && dy < 0)
			{
				orient = 270 - orient;
			}

		}
		else
			orient = 0;

		glRotatef(orient, 0, 0, -1);

		if (it->type == 0)
		//draw ice between nodes;
		{
			glPushMatrix();
			glRotatef(90, 1.0, 0.0, 0.0);
			if (nodes.size() == 1)
			{
				gluCylinder(mySphere, scale * it->scale[0]/32, 0, it->scale[0]/1.5, 4, 1);
				frac3(it, scale/2.5, 0);
				glPushMatrix();
				glRotatef(180, 0.0, 1.0, 0.0);
				gluCylinder(mySphere, scale * it->scale[0]/32, 0, it->scale[0]/1.5, 4, 1);
				frac3(it, scale/2.5, 0);
				glPopMatrix();
			}
			else if (nodes.size() > 0)
			{
				for(int i = 0; i < nodes.size(); i ++)
				{
					if (i == 1)
					{
						gluCylinder(mySphere, scale * it->scale[0]/32, 0, it->scale[0]/1.5, 4, 1);
						frac3(it,scale/2.5, 0);
						glPushMatrix();
						glRotatef(180, 0.0, 1.0, 0.0);
						gluCylinder(mySphere, scale * it->scale[0]/32, 0, it->scale[0]/1.5, 4, 1);
						frac3(it,scale/2.5, 0);
						glPopMatrix();
					}
					else
					{
						if (distance(last, it) < it->maxScale && last->type == 0 && it->maxScale < .3)
						{	
							gluCylinder(mySphere, scale * it->scale[0]/32, scale * last->scale[0]/32, distance(last, it), 4, 1);
							
							glPushMatrix();
							glRotatef(180, 0.0, 1.0, 0.0);
							gluCylinder(mySphere, scale * it->scale[0]/32, 0, it->scale[0]/2, 4, 1);
							frac3(it, scale/2.5, 0);
							glPopMatrix();
						}
						else
						{	
							gluCylinder(mySphere, scale * it->scale[0]/32, 0, it->scale[0]/2, 4, 1);
							frac3(it, scale/2.5, 0);

							glPushMatrix();
							glRotatef(180, 0.0, 1.0, 0.0);
							gluCylinder(mySphere, scale * it->scale[0]/32, 0, it->scale[0]/2, 4, 1);
							frac3(it, scale/2.5, 0);
							glPopMatrix();
						}
					}
					
				}
			}
			glPopMatrix();
			if (draw_branches)
			{
				if (crystal_mode == 0)
					frac2(it, scale);
				else if(crystal_mode == 1)
					frac4(it, scale);
			}
		}
		else if (it->type == 1)
		{
			frac(it, 2* scale);
		}


		//glutSolidCone(i_scale * it->scale/8, i_scale * it->scale , 4, 1);
		glPopMatrix();		

		//placed grow function here for efficiency
		if (!done_growing && it->scale[(int)it->getRecurs()] < it->maxScale)
		{
			//printf("growing\n");

			for (int i = 0; i < it->getRecurs() + 1; i++)
			{
				incr = deg * (it->maxScale - it->scale[i]);
				if (incr < 0.0001)
				{
					incr = 0.0001;
				}

				it->scale[i] += incr;
			}
		}
	}


	//float size = nodes.size();
	//printf("~~~~~~~~~~~~~~~~~~~~Node Size = %f\n", size);
}

void drawNodes(){

	//glColor3f(1.0, 1.0, 1.0);
	//glBindTexture(GL_TEXTURE_2D, 3);
	materials(surface2);

		
for (int i = 0; i < nodes.size(); i++)
	{
		it = nodes.at(i);

		glPushMatrix();
		//draw the glut sphere behind the mesh

		glTranslatef(it->trans.x, it->trans.y, it->trans.z);
		//printf("phi %f", phi);
		glRotatef(phi, -1, 0, 0);
		//glutSolidSphere(it->scale, 10, 10);

		gluSphere(mySphere, it->scale[0], 6, 6);
		glPopMatrix();
		

		//placed grow function here for efficiency
		if (!done_growing && it->scale[0] < it->maxScale)
		{
			//printf("growing\n");
			incr = deg * (it->maxScale - it->scale[0]);
			if (incr < 0.0001)
			{
				incr = 0.0001;
			}

			it->scale[0] += incr;
		}
	}

	//float size = nodes.size();
	//printf("~~~~~~~~~~~~~~~~~~~~Node Size = %f\n", size);
}

void idle(){
	//printf("idling\n");

	//increases max scale of current node if mouse is held down in the same place
	if (mouse_down && mode == 0)
	{
		counter += 0.0001;
		cur_n->maxScale += counter;
	}
	//increase scales of all nodes till they reach maximums
	//grow();
	if (!done_growing)
	{
		//printf("nodes vector size = %f\n",nodes.size() + 0.0);
		glutPostRedisplay();
	}
}

void reset_rotate()
{
	glPushMatrix();
	glLoadIdentity();
	glGetFloatv(GL_MODELVIEW_MATRIX, trackballM);
	glPopMatrix();
	glutPostRedisplay();
}

toggle_blending()
{
	if (!blend_on)
	{
		glDisable(GL_BLEND);
	}
	else
	{
		glEnable(GL_BLEND);
	}
	glutPostRedisplay();

}

void undo()
{
		nodes.pop_back();
		if (nodes.size() > 0)
		{
			cur_n = (Node *)nodes.back();
		}
		else
		{
			cur_n = new Node();
			cur_n->last = new Node();
			last_n = cur_n->last;
		}
		glutPostRedisplay();
}

void reset()
{	
	mode = 0;
	blend_on = false;
	toggle_blending();
	reset_rotate();
	nodes.clear();
	glutPostRedisplay();
}

void display() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	  glMatrixMode(GL_MODELVIEW);
	  if (!mouse_down)
	  {
		  //g_theta = 0;
	  }
	  if (mode == 1 && mouse_down)
	  {
		  rotate();

			startx = endx;
			starty = endy;
			glPushMatrix();
			glLoadIdentity();
			glRotatef(g_theta, rx, ry, rz);

			g_theta = 0;

			glMultMatrixf(trackballM);
			glGetFloatv(GL_MODELVIEW_MATRIX, trackballM);
			glPopMatrix();
	  }

	//check if growing still needed
	if (nodes.size() != 0)
	{
		//done_growing = !(last_n->scale <= last_n->maxScale);
		done_growing = !(nodes.at(nodes.size()-1)->scale[(int)nodes.at(nodes.size()-1)->getRecurs()] <= nodes.at(nodes.size()-1)->maxScale);
	}

  glPushMatrix();
  glLoadIdentity();

  gluLookAt(0.0, 0.0, 6.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
  glGetFloatv(GL_MODELVIEW_MATRIX, camM);
  glPopMatrix();

  draw_background();

  glPushMatrix();
  glLoadIdentity();
  //set up the camera

  glMultMatrixf(camM);
  
  glMultMatrixf(trackballM);
  draw_plane();
  drawIce();
  //drawNodes();
  //drawNodePoints();
  glPopMatrix();
  
  glutSwapBuffers();
}

//the mouse button callback
void mouse(int button, int state, int x, int y) {
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) { /* if the left button is clicked*/
		printf("mouse clicked at %d %d\n", x, GH-1-y);
        
		//rotation stuff
		startx = x;
        starty = GH-y-1;
		
		endx = x;
		endy = GH-y-1;

		mouse_down = true;
		counter = 0.0;
		cur_x = x;
		cur_y = y;

		if (mode == 0)
		{
			Node * n = new Node(x, GH-1-y);
			if (n->trans.x >= -1.5 && n->trans.x <= 1.5 && n->trans.y >= -1.5 && n->trans.y <= 1.5)
			{
				if (mode == 0)
				nodes.push_back(n);
			}
			else
			{
				delete n;
			}
			if (nodes.size() >=1)
			{
				last_n = cur_n;
			}
			cur_n = n;
			cur_n->setLast(last_n);

			idle();
		}
		
		glutPostRedisplay();
	}
		if (state == GLUT_UP)
		{
	        shape_done = true;
			mouse_down = false;
		}

	}

}

Node *last = new Node();
//the mouse move callback
void mouseMove(int x, int y) {

	mouse_down = true;
	endx = x;
    endy = GH-y-1;

	
		Node * n = new Node(x, GH-1-y);

		last_n = cur_n;

	if(mode == 0)
	{

		if (cur_x != x || cur_y != y)
		{
			counter = 0.0;
			cur_x = x;
			cur_y = y;
			cur_n = n;
			cur_n->setLast(last);
			//printf("mouse moved at %d %d\n", x, GH-1-y);


	//keeps nodes close together
			if (distance(last, n) > 0.05
							&& n->trans.x >= -1.5 
							&& n->trans.x <= 1.5 
							&& n->trans.y >= -1.5 
							&& n->trans.y <= 1.5)
			{
				nodes.push_back(n);
				last = cur_n;
			}
			else
			{
				cur_n = last;
				delete n;
			}

		idle();
		//mouse_down = false;
		}
		else
		{
			idle();
		}
	}
	glutPostRedisplay();

}

//the keyboard callback
void keyboard(unsigned char key, int x, int y ){
  switch( key ) {

	  case '1':
		  depth = .49;
		  glutPostRedisplay();
		  break;

	  case '2':
		  depth = .24;
		  glutPostRedisplay();
		  break;
	  case '3':
		  depth = .12;
		  glutPostRedisplay();
		  break;
	  case '4':
		depth = .06;
		glutPostRedisplay();
		break;

	  case 'z': case 'Z':
		  undo();
		  break;

	  case 'y': case 'Y':
		  ice_mode = 0;
		  mode = 0;
		  reset_rotate();
		  glutPostRedisplay();
		  break;
	  case 'u': case 'U':
		  ice_mode = 1;
		  mode = 0;
		  reset_rotate();
		  glutPostRedisplay();
		  break;

	case 'd': case 'D':
		 crystal_mode = !crystal_mode;
		 glutPostRedisplay();
		 break;

    case 'b': case 'B':
		  draw_branches = !draw_branches;
		  glutPostRedisplay();
		  break;

	case 'c': case 'C':
		reset();
		break;

	case 'r': case 'R':
		mode = 1;
		glutPostRedisplay();
		break;

	case 'o': case 'O':
		blend_on = false;
		toggle_blending();
		break;

	case 't': case 'T':
		blend_on = true;
		toggle_blending();
		break;

	case 's': case 'S':
		surface_texture = !surface_texture;
		glutPostRedisplay();
		break;
    case 'q': case 'Q' :
      exit( EXIT_SUCCESS );
      break;
  }
  //settings_change = true;
}


void reshape(int w, int h) {
  GW = w;
  GH = h;
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  //if (view_mode == 0)
//    glOrtho( -(float)w/h, (float)w/h, -1.0, 1.0, 1.0, 15.0);
  //else... fill in
  //else
	gluPerspective(30.0, (float)w/h, 1.0, 15.0);
  glMatrixMode(GL_MODELVIEW);
  glViewport(0, 0, w, h);
  
  glutPostRedisplay();
  glutPostRedisplay();
  
}

void crystal_menu(int value)
{
	if (value == 1)
	{
		crystal_mode = 0;
		draw_branches = 0;
	}
	else if (value == 2)
	{
		crystal_mode = 1;
		draw_branches = 1;
	}
	else if (value == 3)
	{
		crystal_mode = 0;
		draw_branches = 1;
	}
	
	glutPostRedisplay();
}

void color_menu(int value)
{
	if (value == 0) cur_col = 7;
	else if (value == 1) cur_col = 3;
	else if (value == 2) cur_col = 4;
	else if (value == 3) cur_col = 5;
	else if (value == 4) cur_col = 6;
}

void fractal_menu(int value)
{
	if (value == 1) depth = .49;
	else if (value == 2) depth = .24;
	else if (value == 3) depth = .12;
	else if (value == 4) depth = .05;

	glutPostRedisplay();
}

void mymenu (int value)
{
	if (value == 6) exit(0);
	else if (value == 0) reset();
	else if (value == 1)
	{
		//draw crystals
		reset_rotate();
		mode = 0;
		ice_mode = 0;
		printf("Draw Crystals Mode\n");
	}
	else if (value == 2)
	{
		printf("Rotate Mode\n");
		mode = 1;
	}
	else if (value == 3)
	{
		blend_on = false;
		toggle_blending();
	}
	else if (value == 4)
	{
		blend_on = true;
		toggle_blending();
	}
	else if (value == 5)
	{	
		//draw chunks
		reset_rotate();
		mode = 0;
		ice_mode = 1;
		printf("Draw Chunks Mode\n");
	}
	else if (value == 6)
	{
		ice_mode = 1;
	}
	else if (value == 7)
	{
		crystal_mode = !crystal_mode;
	}
	else if (value == 8)
	{
		undo();
	}
	else if (value == 9)
	{
		surface_texture = !surface_texture;
	}


	glutPostRedisplay();
}

//initialization for texture mapping
void init_tex() {
  
  glBindTexture(GL_TEXTURE_2D, 0);
  glTexImage2D(GL_TEXTURE_2D, 0, 3, 64, 64, 0, GL_RGB, GL_UNSIGNED_BYTE, pixel);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
}

//routines to load in a bmp files - must be 2^nx2^m and a 24bit bmp
GLvoid LoadTexture(char* image_file, int texID) { 
  
  TextureImage = (Image *) malloc(sizeof(Image));
  if (TextureImage == NULL) {
    printf("Error allocating space for image");
    exit(1);
  }
  cout << "trying to load " << image_file << endl;
  if (!ImageLoad(image_file, TextureImage)) {
    exit(1);
  }  
  /*  2d texture, level of detail 0 (normal), 3 components (red, green, blue),            */
  /*  x size from image, y size from image,                                              */    
  /*  border 0 (normal), rgb color data, unsigned byte data, data  */ 
  glBindTexture(GL_TEXTURE_2D, texID);
  
  /*glTexImage2D(GL_TEXTURE_2D, 0, 3,
    TextureImage->sizeX, TextureImage->sizeY,
    0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->data);
	*/

//enable mipmaps
gluBuild2DMipmaps(GL_TEXTURE_2D, 3,
    TextureImage->sizeX, TextureImage->sizeY,
    GL_RGB, GL_UNSIGNED_BYTE, TextureImage->data);


  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); /*  cheap scaling when image bigger than texture */    
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); /*  cheap scaling when image smalled than texture*/
  
}

/* BMP file loader loads a 24-bit bmp file only */

/*
* getint and getshort are help functions to load the bitmap byte by byte
*/
static unsigned int getint(FILE *fp) {
  int c, c1, c2, c3;
  
  /*  get 4 bytes */ 
  c = getc(fp);  
  c1 = getc(fp);  
  c2 = getc(fp);  
  c3 = getc(fp);
  
  return ((unsigned int) c) +   
    (((unsigned int) c1) << 8) + 
    (((unsigned int) c2) << 16) +
    (((unsigned int) c3) << 24);
}

static unsigned int getshort(FILE *fp){
  int c, c1;
  
  /* get 2 bytes*/
  c = getc(fp);  
  c1 = getc(fp);
  
  return ((unsigned int) c) + (((unsigned int) c1) << 8);
}

/*  quick and dirty bitmap loader...for 24 bit bitmaps with 1 plane only.  */

int ImageLoad(char *filename, Image *image) {
  FILE *file;
  unsigned long size;                 /*  size of the image in bytes. */
  unsigned long i;                    /*  standard counter. */
  unsigned short int planes;          /*  number of planes in image (must be 1)  */
  unsigned short int bpp;             /*  number of bits per pixel (must be 24) */
  char temp;                          /*  used to convert bgr to rgb color. */
  
  /*  make sure the file is there. */
  if ((file = fopen(filename, "rb"))==NULL) {
    printf("File Not Found : %s\n",filename);
    return 0;
  }
  
  /*  seek through the bmp header, up to the width height: */
  fseek(file, 18, SEEK_CUR);
  
  /*  No 100% errorchecking anymore!!! */
  
  /*  read the width */    image->sizeX = getint (file);
  
  /*  read the height */ 
  image->sizeY = getint (file);
  
  /*  calculate the size (assuming 24 bits or 3 bytes per pixel). */
  size = image->sizeX * image->sizeY * 3;
  
  /*  read the planes */    
  planes = getshort(file);
  if (planes != 1) {
    printf("Planes from %s is not 1: %u\n", filename, planes);
    return 0;
  }
  
  /*  read the bpp */    
  bpp = getshort(file);
  if (bpp != 24) {
    printf("Bpp from %s is not 24: %u\n", filename, bpp);
    return 0;
  }
  
  /*  seek past the rest of the bitmap header. */
  fseek(file, 24, SEEK_CUR);
  
  /*  read the data.  */
  image->data = (char *) malloc(size);
  if (image->data == NULL) {
    printf("Error allocating memory for color-corrected image data");
    return 0; 
  }
  
  if ((i = fread(image->data, size, 1, file)) != 1) {
    printf("Error reading image data from %s.\n", filename);
    return 0;
  }
  
  for (i=0;i<size;i+=3) { /*  reverse all of the colors. (bgr -> rgb) */
    temp = image->data[i];
    image->data[i] = image->data[i+2];
    image->data[i+2] = temp;
  }
  
  fclose(file); /* Close the file and release the filedes */
  
  /*  we're done. */
  return 1;
}



int main( int argc, char** argv ){

	surface_texture = 0;

	cur_col = 7;

	//seeding random number generator
	srand( (unsigned)time( NULL ) );
	draw_branches = true;

	//Node *n = new Node();
	cur_n = new Node();
	last_n = new Node();

	int menu1, menu2, menu3, menu4;
	glutInit( &argc, argv );

  //intializations

	//rotation stuff

	startx = endx = 0;
	starty = endy = 0;
	  rx = ry = rz = 0.0;
      g_theta = 0.0;
  
	  mode = 0;
	  ice_mode = 1;
	  crystal_mode = 1;

	//double buffer
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
    glutIdleFunc(idle);
	glutInitWindowSize(600, 600);
	glutInitWindowPosition(200, 200);
	glutCreateWindow( "Ice" );
	glClearColor(0.0, 0.0, 0.0, 1.0);
  //global variable intialization
  GW = GH = 400;
 
  counter = 0.0;
  mouse_down = false;
  cur_x = cur_y = 0.0;
  done_growing = true;


  //register the callback functions
  //glBlendFunc(
	glutDisplayFunc( display );
	glutReshapeFunc( reshape );
  glutMouseFunc( mouse );
  glutMotionFunc( mouseMove );
  glutKeyboardFunc( keyboard );


  //transparency stuff
  glBlendFunc(GL_SRC_ALPHA,  GL_ONE_MINUS_SRC_ALPHA);
  blend_on = false;
  //glEnable(GL_BLEND);


    //enable z-buffer
  glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
  init_lighting();

  //load textures
  glEnable(GL_TEXTURE_2D);
  init_tex();
   LoadTexture("background.bmp", 1);
   LoadTexture("white.bmp", 2);
   LoadTexture("ice.bmp", 3);
   LoadTexture("ice2.bmp", 4);
   LoadTexture("ice3.bmp", 5);
   LoadTexture("ice4.bmp", 6);
   LoadTexture("ice5.bmp", 7);
   LoadTexture("snow.bmp", 8);
  
  //set-up viewing
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-1.0, 1.0, -1.0, 1.0, 1.0, 15.0);
  glMatrixMode(GL_MODELVIEW);
  glViewport(0, 0, 600, 600); 

  // menu

  menu4 = glutCreateMenu(color_menu);
  glutAddMenuEntry("White", 0);
  glutAddMenuEntry("Blue", 1);
  glutAddMenuEntry("Green", 3);
  glutAddMenuEntry("Red", 2);
  glutAddMenuEntry("Purple", 4);

  menu3 = glutCreateMenu(crystal_menu);
  glutAddMenuEntry("1D", 1);
  glutAddMenuEntry("2D", 2);
  glutAddMenuEntry("3D", 3);

  menu2 = glutCreateMenu(fractal_menu);
  glutAddMenuEntry("0", 1);
  glutAddMenuEntry("1", 2);
  glutAddMenuEntry("2", 3);
  //glutAddMenuEntry("3", 4);

  menu1 = glutCreateMenu(mymenu);
  //glutAddMenuEntry("Toggle Crystal Type", 7);
  glutAddMenuEntry("Draw Crystals (Y)", 1);
  glutAddMenuEntry("Draw Chunks (U)", 5);
  glutAddSubMenu("Ice Color", menu4);
  glutAddMenuEntry("Opaque (O)", 3);
  glutAddMenuEntry("Translucent (T)", 4);
  glutAddMenuEntry("Toggle Surface Texture (S)", 9);
  glutAddMenuEntry("Rotate Scene (R)", 2);
  glutAddSubMenu("Crystal Complexity", menu3);
  glutAddSubMenu("Fractal Depth", menu2);
  glutAddMenuEntry("Undo (Z)", 8);
  glutAddMenuEntry("Clear (C)", 0);
  glutAddMenuEntry("Exit (Q)", 6);
  glutAttachMenu(GLUT_RIGHT_BUTTON);


    //create a sphere
  mySphere = gluNewQuadric();
  gluQuadricDrawStyle(mySphere, GLU_FILL);
  //make opengl generate the texture coordinates for the sphere
  gluQuadricTexture(mySphere, GL_TRUE);

	glutMainLoop();
}